# -*- coding: utf-8 -*-
#!/usr/bin/python

from geopy.geocoders import Nominatim
from mr_logger.logger import Logger
from keyboard import KeyboardBuilder
geolocator = Nominatim()
logger = Logger()


def locate(location):
    result = None
    try:
        result = geolocator.geocode(str(location))
    except Exception as e:
        logger.warn('Can not get location %s in maxmind db;\n%s' % (location, e))
    return result


class GeoValidator:
    """ class for validate city throw pygeo and sending response """
    sender = None

    def __init__(self, sender):
        self.sender = sender

    #def ehlo(self):

    def validate(self, incoming_city, lang, chat_id):
        """ this method will validate address with pygeo """
        validated = locate(incoming_city)
        if lang is 'ru':
            text = 'К сожалению город не найден, попробуйте еще раз.'
            btn_caption_yes = 'Да'
            btn_caption_no = 'Нет'
        else:
            text = 'Sorry, i can not find this city. Please try again.'
            btn_caption_yes = 'Yes'
            btn_caption_no = 'No'
        if validated:
            buttons = [[btn_caption_yes, btn_caption_no]]
            callbacks = [['city_ok|%s' % chat_id, 'city_false|%s' % chat_id]]
            kb = KeyboardBuilder()
            keyboard = kb.get_inline_keyboard(keyboard=buttons, callbacks=callbacks)
            self.sender.send_message(chat_id=chat_id, text=str(validated), reply_markup=keyboard)
            return True
        else:
            self.sender.send_message(chat_id=chat_id, text=text)
            return False


# -*- coding: utf-8 -*-
#!/usr/bin/python
from telegram import InlineKeyboardButton
from telegram import InlineKeyboardMarkup
from telegram import ReplyKeyboardMarkup
import emoji
import re
from mr_logger.logger import Logger


class KeyboardBuilder:
    """ keyboard incoming data:
        [
             ['1<emo_code>'], ['2'], ['3'],
             ['4', '5'], ['6'],
             ['7'], ['8', '9']
        ]
    """

    def __init__(self):
        self.logger = Logger()

    @staticmethod
    def get_emo(code):
        """ get emotion unicode """
        emo = emoji.emojize(code, use_aliases=True)
        return emo

    def _basic_prepare(self, keyboard):
        """ change <code> to emoji in buttons  """
        result_keyboard = []
        for line in keyboard:
            btn_line = []
            for button in line:
                emo = re.search(r'<.*?>', button)
                if emo:
                    marker = emo.group(0)
                    emo_code = marker.replace('<', '').replace('>', '')
                    button = button.replace(marker, self.get_emo(emo_code))
                btn_line.append(button)
            result_keyboard.append(btn_line)
        return result_keyboard

    def _inline_prepare(self, keyboard, callbacks, urls=None):
        """ create inline keyboard from list of list """
        inline_keyboard = []
        for ind_line, line in enumerate(keyboard):
            btn_line = []
            for ind_btn, button in enumerate(line):
                if not urls:
                    button = InlineKeyboardButton(text=button,
                                                  callback_data=callbacks[ind_line][ind_btn])
                else:
                    button = InlineKeyboardButton(text=button,
                                                  callback_data=callbacks[ind_line][ind_btn],
                                                  url=urls[ind_line][ind_btn])
                btn_line.append(button)
            inline_keyboard.append(btn_line)
        return inline_keyboard

    def get_keyboard(self, keyboard, is_one_time, is_selective):
        """ basic method for get reply keyboard """
        keyboard = self._basic_prepare(keyboard=keyboard)
        return ReplyKeyboardMarkup(keyboard=keyboard, one_time_keyboard=is_one_time,
                    selective=is_selective, resize_keyboard=True)

    def get_inline_keyboard(self, keyboard, callbacks):
        """ method for get inline keyboard """
        keyboard = self._basic_prepare(keyboard)
        inline_keyboard = self._inline_prepare(keyboard=keyboard, callbacks=callbacks)
        return InlineKeyboardMarkup(inline_keyboard=inline_keyboard)



# -*- coding: utf-8 -*-
#!/usr/bin/python

import traceback
from mr_logger.logger import Logger
from telegram.error import BadRequest, Unauthorized

logger = Logger()


class Sender:
    """ this class will send bot messages (photo, voice, video, audio, docs) """
    """ bot is instance if Telegram bot """
    """ return True on success and false on error """

    bot = None

    def __init__(self, bot):
        self.bot = bot

    def edit_message_text(self, message_id, chat_id, text, reply_markup, parse_mode='HTML'):
        try:
            self.bot.editMessageText(message_id=message_id, chat_id=chat_id,
                                                            text=text, reply_markup=reply_markup,
                                                            parse_mode=parse_mode)
            return True
        except BadRequest as bad_request_exception:
            logger.warn(traceback.format_exception_only(type(bad_request_exception),
            bad_request_exception))
        except Unauthorized:
            logger.warn('[Unauthorized]: cant edit message text.\nID=%s' % str(message_id))
        except Exception as e:
            logger.err(e)
        return False


    def edit_inline_message_text(self, inline_message_id, text, reply_markup):
        try:
            self.bot.editMessageText(inline_message_id=inline_message_id,
                                                            text=text, reply_markup=reply_markup)
            return True
        except BadRequest as bad_request_exception:
            logger.warn(traceback.format_exception_only(type(bad_request_exception),
            bad_request_exception))
        except Unauthorized:
            logger.warn('[Unauthorized]: cant edit message text.\nID=%s' % str(inline_message_id))
        except Exception as e:
            logger.err(e)
        return False


    # todo venue
    def send_message(self, chat_id, parse_mode=None, text=None, photo=None, video=None, document=None,
                     voice=None, audio=None, sticker=None, location=None, contact=None,
                     reply_markup=None, reply_to_message_id=None):
        try:
            if text:
                if not parse_mode:
                    self.bot.sendMessage(chat_id=chat_id,
                                    text=text, reply_markup=reply_markup,
                                    reply_to_message_id=reply_to_message_id)
                else:
                    self.bot.sendMessage(chat_id=chat_id,
                                    text=text, reply_markup=reply_markup,
                                    reply_to_message_id=reply_to_message_id, parse_mode=parse_mode)
            elif photo:
                self.bot.sendPhoto(chat_id=chat_id, photo=photo, reply_markup=reply_markup,
                                   reply_to_message_id=reply_to_message_id)
            elif video:
                self.bot.sendVideo(chat_id=chat_id, video=video, reply_markup=reply_markup,
                                   reply_to_message_id=reply_to_message_id)
            elif document:
                self.bot.sendDocument(chat_id=chat_id, document=document, reply_markup=reply_markup,
                                   reply_to_message_id=reply_to_message_id)
            elif voice:
                self.bot.sendVoice(chat_id=chat_id, voice=voice, reply_markup=reply_markup,
                                   reply_to_message_id=reply_to_message_id)
            elif audio:
                self.bot.sendAudio(chat_id=chat_id, audio=audio, reply_markup=reply_markup,
                                   reply_to_message_id=reply_to_message_id)
            elif sticker:
                self.bot.sendSticker(chat_id=chat_id, sticker=sticker, reply_markup=reply_markup,
                    reply_to_message_id=reply_to_message_id)
            elif location:
                self.bot.sendLocation(chat_id=chat_id, latitude=location.latitude,
                                      longitude=location.longitude, reply_markup=reply_markup,
                                      reply_to_message_id=reply_to_message_id)
            elif contact:
                self.bot.sendContact(chat_id=chat_id, phone_number=contact.phone_number,
                                     first_name=contact.first_name, reply_markup=reply_markup,
                                     reply_to_message_id=reply_to_message_id)
            return True
        except BadRequest as bad_request_exception:
            logger.warn(traceback.format_exception_only(type(bad_request_exception),
            bad_request_exception))
        except Unauthorized:
            logger.warn('cant send message to chat_id %s ( the bot was blocked by the user )'
            % str(chat_id))
        except Exception as e:
            logger.err(e)
        return False



# -*- coding: utf-8 -*-
from keyboard import KeyboardBuilder


# todo generate answer - choose answer by
class Answer:

    def __init__(self):
        self.kb = KeyboardBuilder()

    def _create_reply_keyboard(self, keyboard, is_one_time, is_selective):
        return self.kb.get_reply_keyboard(keyboard=keyboard,
                    is_one_time=is_one_time, is_selective=is_selective)

    def generate_answer():
        pass


class Responder:

    def __init__(self, sender):
        self.sender = sender

    def respond(self):
        print(self.kb)



# -*- coding: utf-8 -*-

#from distutils.core import setup
from setuptools import setup

setup(
  name='mr_bot',
  packages=['mr_bot'],
  version='1.1.0',
  description='Small package for assist telegram bot development',
  author='Root Kid',
  author_email='shaman@born2fish.ru',
  url='https://github.com/r00tkid/mr_bot.git',
  download_url='https://github.com/r00tkid/mr_bot/tarball/1.1.0',
  keywords=['support', 'telegram', 'bot', 'keyboard'],  # keywords
  classifiers=[],
  install_requires=[
      "emoji",
      'colored',
      'future',
      'mr-logger',
      'python-telegram-bot',
      'termcolor',
      'geopy',
      'urllib3'
  ],
)




# -*- coding: utf-8 -*-
#!/usr/bin/python
from mr_bot.sender import Sender
from mr_bot.keyboard import KeyboardBuilder

DEFAULT_LANGUAGE = 'EN'
kb = KeyboardBuilder()


def send_select_language_keyboard(bot, user_id):
    sender = Sender(bot=bot)
    message_text = "Please select your language:"
    keyboard = [['<:earth_asia:>Russian', '<:earth_americas:>English']]
    keyboard = kb.get_keyboard(keyboard=keyboard, is_one_time=True, is_selective=True)
    sender.send_message(chat_id=user_id, text=message_text, reply_markup=keyboard)


def send_reponse_on_language_select_action(lang, user_id, bot, keyboard):
    sender = Sender(bot=bot)
    if lang == 'ru':
        text = 'Я сохранил ваши языковые настройки.'
    elif lang == 'en':
        text = 'I have saved your language settings.'
    sender.send_message(chat_id=user_id, text=text, reply_markup=keyboard)


def send_validate_city_keyboard(lang, user_id, bot, validated):
    """ validated is None or Location (pygeo) """
    sender = Sender(bot=bot)
    if lang == 'ru':
        text = 'К сожалению город не найден, попробуйте еще раз.'
        btn_caption_yes = 'Да'
        btn_caption_no = 'Нет'
    else:
        text = 'Sorry, i can not find this city. Please try again.'
        btn_caption_yes = 'Yes'
        btn_caption_no = 'No'
    if validated:
        buttons = [[btn_caption_yes, btn_caption_no]]
        callbacks = [['city_ok|%s' % user_id, 'city_false|%s' % user_id]]
        keyboard = kb.get_inline_keyboard(keyboard=buttons, callbacks=callbacks)
        sender.send_message(chat_id=user_id, text=str(validated), reply_markup=keyboard)
        return True
    else:
        sender.send_message(chat_id=user_id, text=text)
        return False


def send_manage_folders_keyboard(lang, user_id, bot):
    sender = Sender(bot=bot)
    if lang == 'ru':
        message_text = "Перешел в раздел управления папками"
        keyboard = [['<:file_folder:> Добавить папку', '<:pouch:> Список папок'],
        ['<:back:> Назад']]
    else:
        message_text = "We are in the folders manage section"
        keyboard = [['<:file_folder:> Add folder', '<:pouch:> Folders list'], ['<:back:>']]

    keyboard = kb.get_keyboard(keyboard=keyboard, is_one_time=False, is_selective=True)
    sender.send_message(chat_id=user_id, text=message_text, reply_markup=keyboard)
